@@ -1,15 +1,47 @@
+# pylint: disable=line-too-long,multiple-statements
 from contextlib import contextmanager
+from typing import TypeVar, Generic, AsyncIterator, Tuple, ContextManager, Callable, overload
 
 import trio
 
 try:
-    from trio.lowlevel import ParkingLot as WaitQueue
-except ImportError:
-    from trio.hazmat import ParkingLot as WaitQueue
+    from trio import lowlevel
+except ImportError:  # pragma: no cover
+    from trio import hazmat as lowlevel  # type: ignore
 
 from ._ref_counted_default_dict import _RefCountedDefaultDict
 
 
+class _WaitQueue:
+    """Fast replacement for trio.ParkingLot.
+
+    The typical unpark_all() is 2x faster.
+    """
+
+    __slots__ = ['tasks']
+
+    def __init__(self):
+        self.tasks = set()
+
+    def __len__(self):
+        return len(self.tasks)
+
+    async def park(self):
+        task = lowlevel.current_task()
+        self.tasks.add(task)
+
+        def abort_fn(_):
+            self.tasks.remove(task)
+            return lowlevel.Abort.SUCCEEDED
+
+        await lowlevel.wait_task_rescheduled(abort_fn)
+
+    def unpark_all(self):
+        for task in self.tasks:
+            lowlevel.reschedule(task)
+        self.tasks.clear()
+
+
 def _ANY_TRANSITION(value, old_value):
     return True
 
@@ -19,13 +51,17 @@ def _ANY_VALUE(value):
 
 
 _NONE = object()
+T = TypeVar('T')
+T_OUT = TypeVar('T_OUT')
+P = Callable[[T], bool]
+P2 = Callable[[T, T], bool]
 
 
 class _Result:
     __slots__ = ['event', 'value']
 
     def __init__(self):
-        self.event = WaitQueue()
+        self.event = _WaitQueue()
         self.value = None
 
 
@@ -60,7 +96,7 @@ class _ValueWrapper:
         return x == self.value
 
 
-class AsyncValue:
+class AsyncValue(Generic[T]):
     """
     Value wrapper offering the ability to wait for a value or transition.
 
@@ -76,13 +112,19 @@ class AsyncValue:
         >>> # wait for value match by predicate
         >>> await a.wait_value(lambda v: v > 10)
         >>> ...
+        >>> # values via iteration (with eventual consistency)
+        >>> async for value a.eventual_values(lambda v: v > 10):
+        >>>     ...
+        >>> # wait for any transition
+        >>> await a.wait_transition()
+        >>> ...
         >>> # wait for transition by equality
         >>> await a.wait_transition(14)
         >>> ...
-        >>> # wait for transition by predicate (default: any transition)
+        >>> # wait for transition by predicate
         >>> await a.wait_transition(lambda v, old: v > 10 and old < 0)
         >>> ...
-        >>> # repeated transitions via iteration
+        >>> # repeated transitions via iteration (misses values while blocked)
         >>> async for value, _ in a.transitions(lambda v, old: v > 10 and old < 0):
         >>>     ...
 
@@ -111,7 +153,7 @@ class AsyncValue:
     predicates.
     """
 
-    def __init__(self, value):
+    def __init__(self, value: T):
         self._value = value
         self._level_results = _RefCountedDefaultDict(_Result)
         self._edge_results = _RefCountedDefaultDict(_Result)
@@ -119,15 +161,15 @@ class AsyncValue:
         self._transforms = _RefCountedDefaultDict(lambda: AsyncValue(_NONE))
 
     def __repr__(self):
-        return f"{self.__class__.__name__}(value={self.value})"
+        return f"{self.__class__.__name__}({self.value})"
 
     @property
-    def value(self):
+    def value(self) -> T:
         """The wrapped value"""
         return self._value
 
     @value.setter
-    def value(self, x):
+    def value(self, x: T):
         if self._value != x:
             old = self._value
             new = self._value = x
@@ -148,7 +190,11 @@ class AsyncValue:
             await result.event.park()
         return result.value
 
-    async def wait_value(self, value_or_predicate, *, held_for=0):
+    @overload
+    async def wait_value(self, value: T, *, held_for=0.) -> T: ...
+    @overload
+    async def wait_value(self, predicate: P, *, held_for=0.) -> T: ...
+    async def wait_value(self, value_or_predicate, *, held_for=0.):
         """
         Wait until given predicate f(value) is True.
 
@@ -178,6 +224,13 @@ class AsyncValue:
             break
         return value
 
+    # NOTE: it's hard to make both type check and lint happy with generator overloads
+    # https://github.com/python/mypy/issues/10301
+    # https://github.com/PyCQA/astroid/issues/1015
+    @overload
+    async def eventual_values(self, value: T) -> AsyncIterator[T]: yield self._value
+    @overload
+    async def eventual_values(self, predicate: P = _ANY_VALUE) -> AsyncIterator[T]: yield self._value
     async def eventual_values(self, value_or_predicate=_ANY_VALUE):
         """
         Yield values matching the predicate with eventual consistency
@@ -194,7 +247,7 @@ class AsyncValue:
         the given value.
         """
         predicate = _ValueWrapper(value_or_predicate)
-        last_value = None
+        last_value = self._value
         with self._level_results.open_ref(predicate) as result, \
                 self._level_results.open_ref(lambda v: v != last_value) as not_last_value:
             while True:
@@ -207,8 +260,11 @@ class AsyncValue:
                 if self._value == last_value:
                     await not_last_value.event.park()
 
-    # TODO: held_for
     # TODO: implement wait_transition() using transitions()
+    @overload
+    async def wait_transition(self, value: T) -> Tuple[T, T]: ...
+    @overload
+    async def wait_transition(self, predicate: P2 = _ANY_TRANSITION) -> Tuple[T, T]: ...
     async def wait_transition(self, value_or_predicate=_ANY_TRANSITION):
         """
         Wait until given predicate f(value, old_value) is True.
@@ -219,10 +275,19 @@ class AsyncValue:
         If a non-callable is provided, it's equivalent to a predicate matching
         the given value.
 
+        Note that unlike `wait_value()`, it is easy to have race conditions
+        when using `wait_transition()`.  Always consider whether it's possible
+        for the initially desired transition to have already occurred due to
+        task scheduling order, etc.
+
         returns (value, old_value) which satisfied the predicate
         """
         return await self._wait_predicate(self._edge_results, _ValueWrapper(value_or_predicate))
 
+    @overload
+    async def transitions(self, value: T) -> AsyncIterator[Tuple[T, T]]: yield (self._value, self._value)
+    @overload
+    async def transitions(self, predicate: P2 = _ANY_TRANSITION) -> AsyncIterator[Tuple[T, T]]: yield (self._value, self._value)
     async def transitions(self, value_or_predicate=_ANY_TRANSITION):
         """
         Yield (value, old_value) for transitions matching the predicate
@@ -239,6 +304,11 @@ class AsyncValue:
             >>> while True:
             >>>     value, old_value = await async_value.wait_transition(...)
             >>>     ...
+
+        Unlike the `eventual_values()` iterator, use of the `transitions()` is
+        prone to races when entering the loop.  Always consider whether it's
+        possible for the desired transition to have already occurred due to
+        task scheduling order, etc.
         """
 
         # Note this is not simply `while True: await wait_transition(...)`,
@@ -249,8 +319,9 @@ class AsyncValue:
                 await result.event.park()
                 yield result.value
 
-    @contextmanager
-    def open_transform(self, function):
+    # TODO: make the output's `value` read-only somehow
+    def open_transform(self, function: Callable[[T], T_OUT]) \
+            -> ContextManager['AsyncValue[T_OUT]']:
         """Yield a derived AsyncValue with the given transform applied
 
         Synopsis::
@@ -261,7 +332,11 @@ class AsyncValue:
         >>>     x.value = 10
         >>>     assert y.value == 20
         """
-        # TODO: make the output's `value` read-only somehow
+        # type hint workaround for https://youtrack.jetbrains.com/issue/PY-36444
+        return self._open_transform(function)
+
+    @contextmanager
+    def _open_transform(self, function):
         with self._transforms.open_ref(function) as output:
             if output.value is _NONE:
                 output.value = function(self.value)
